@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -128,17 +128,12 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   IndexPacket
     *indexes;
 
-  ssize_t
-    x;
-
   PixelPacket
     *q;
 
-  unsigned char
-    *p;
-
   ssize_t
     length,
+    x,
     y;
 
   struct jbg_dec_state
@@ -147,7 +142,8 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   unsigned char
     bit,
     *buffer,
-    byte;
+    byte,
+    *p;
 
   /*
     Open image file.
@@ -169,6 +165,9 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   /*
     Initialize JBIG toolkit.
   */
+  if ((image->columns != (unsigned long) image->columns) ||
+      (image->rows != (unsigned long) image->rows))
+    ThrowReaderException(ImageError,"WidthOrHeightExceedsLimit");
   jbg_dec_init(&jbig_info);
   jbg_dec_maxsize(&jbig_info,(unsigned long) image->columns,(unsigned long)
     image->rows);
@@ -200,7 +199,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
         count;
 
       status=jbg_dec_in(&jbig_info,p,length,&count);
-      p+=count;
+      p+=(ptrdiff_t) count;
       length-=(ssize_t) count;
     }
   } while ((status == JBG_EAGAIN) || (status == JBG_EOK));
@@ -276,7 +275,10 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   */
   jbg_dec_free(&jbig_info);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 #endif
@@ -417,6 +419,9 @@ static void JBIGEncode(unsigned char *pixels,size_t length,void *data)
 static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   Image *image)
 {
+  const PixelPacket
+    *p;
+
   double
     version;
 
@@ -429,20 +434,12 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   MemoryInfo
     *pixel_info;
 
-  const PixelPacket
-    *p;
-
-  ssize_t
-    x;
-
-  unsigned char
-    *q;
-
   size_t
     number_images,
     number_packets;
 
   ssize_t
+    x,
     y;
 
   struct jbg_enc_state
@@ -451,7 +448,8 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   unsigned char
     bit,
     byte,
-    *pixels;
+    *pixels,
+    *q;
 
   /*
     Open image file.
@@ -495,7 +493,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
       for (x=0; x < (ssize_t) image->columns; x++)
       {
         byte<<=1;
-        if (GetPixelLuma(image,p) < (QuantumRange/2.0))
+        if (GetPixelLuma(image,p) < ((MagickRealType) QuantumRange/2.0))
           byte|=0x01;
         bit++;
         if (bit == 8)
@@ -570,7 +568,8 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
-  (void) CloseBlob(image);
-  return(MagickTrue);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  return(status);
 }
 #endif
