@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -112,24 +112,39 @@ static MagickBooleanType
 %
 */
 static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
-  unsigned char *chroma1,unsigned char *chroma2)
+  unsigned char *chroma1,unsigned char *chroma2,ExceptionInfo *exception)
 {
 #define IsSync(sum)  ((sum & 0xffffff00UL) == 0xfffffe00UL)
 #define PCDGetBits(n) \
 {  \
+  ssize_t \
+    byte_count = 0x800; \
+  \
   sum=(sum << n) & 0xffffffff; \
   bits-=n; \
   while (bits <= 24) \
   { \
     if (p >= (buffer+0x800)) \
       { \
-        count=ReadBlob(image,0x800,buffer); \
+        byte_count=ReadBlob(image,0x800,buffer); \
+        if (byte_count != 0x800) \
+          { \
+            (void) ThrowMagickException(&image->exception,GetMagickModule(), \
+              CorruptImageWarning,"CorruptImage","`%s'",image->filename); \
+            break; \
+          } \
         p=buffer; \
       } \
     sum|=(((unsigned int) (*p)) << (24-bits)); \
     bits+=8; \
     p++; \
   } \
+  if (byte_count != 0x800) \
+    { \
+      (void) ThrowMagickException(&image->exception,GetMagickModule(), \
+        CorruptImageWarning,"CorruptImage","`%s'",image->filename); \
+      break; \
+    } \
 }
 
   typedef struct PCDTable
@@ -148,31 +163,28 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   PCDTable
     *pcd_table[3];
 
-  ssize_t
-    i,
-    j;
-
   PCDTable
     *r;
 
-  unsigned char
-    *p,
-    *q;
-
   size_t
     bits,
-    length,
     plane,
-    pcd_length[3],
     row,
     sum;
 
   ssize_t
     count,
+    i,
+    j,
+    pcd_count,
+    pcd_length[3],
     quantum;
 
   unsigned char
-    *buffer;
+    *buffer,
+    *p,
+    *q,
+    *sentinel;
 
   /*
     Initialize Huffman tables.
@@ -196,11 +208,12 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     pcd_table[i]=(PCDTable *) NULL;
     pcd_length[i]=0;
   }
-  for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
+  pcd_count=(ssize_t) (image->columns > 1536 ? 3 : 1);
+  for (i=0; i < pcd_count; i++)
   {
     PCDGetBits(8);
-    length=(sum & 0xff)+1;
-    pcd_table[i]=(PCDTable *) AcquireQuantumMemory(length,
+    pcd_length[i]=(ssize_t) (sum & 0xff)+1;
+    pcd_table[i]=(PCDTable *) AcquireQuantumMemory((size_t) pcd_length[i],
       sizeof(*pcd_table[i]));
     if (pcd_table[i] == (PCDTable *) NULL)
       {
@@ -211,7 +224,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           image->filename);
       }
     r=pcd_table[i];
-    for (j=0; j < (ssize_t) length; j++)
+    for (j=0; j < pcd_length[i]; j++)
     {
       PCDGetBits(8);
       r->length=(unsigned int) (sum & 0xff)+1;
@@ -229,7 +242,6 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       r->mask=(~((1U << (32-r->length))-1));
       r++;
     }
-    pcd_length[i]=(size_t) length;
   }
   if (EOFBlob(image) == MagickFalse)
     {
@@ -249,9 +261,9 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     Recover the Huffman encoded luminance and chrominance deltas.
   */
   count=0;
-  length=0;
   plane=0;
   row=0;
+  sentinel=luma+30*(image->columns+1)*image->rows;
   for (q=luma; EOFBlob(image) == MagickFalse; )
   {
     if (IsSync(sum) != 0)
@@ -290,21 +302,20 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           }
           default:
           {
-            for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
+            for (i=0; i < pcd_count; i++)
               pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
             buffer=(unsigned char *) RelinquishMagickMemory(buffer);
             ThrowBinaryImageException(CorruptImageError,"CorruptImage",
               image->filename);
           }
         }
-        length=pcd_length[plane];
         continue;
       }
     /*
       Decode luminance or chrominance deltas.
     */
     r=pcd_table[plane];
-    for (i=0; ((i < (ssize_t) length) && ((sum & r->mask) != r->sequence)); i++)
+    for (i=1; ((i < pcd_length[plane]) && ((sum & r->mask) != r->sequence)); i++)
       r++;
     if ((row > image->rows) || (r == (PCDTable *) NULL))
       {
@@ -316,6 +327,8 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           PCDGetBits(1);
         continue;
       }
+    if ((q < luma) || (q >= sentinel))
+      break;
     if (r->key < 128)
       quantum=(ssize_t) (*q)+r->key;
     else
@@ -328,9 +341,11 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   /*
     Relinquish resources.
   */
-  for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
+  for (i=0; i < pcd_count; i++)
     pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  if ((q < luma) || (q >= sentinel))
+    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
   return(MagickTrue);
 }
 
@@ -467,9 +482,9 @@ static void Upsample(const size_t width,const size_t height,
       *q=(unsigned char) ((((size_t) *p)+((size_t) *r)+1) >> 1);
       *(q+1)=(unsigned char) ((((size_t) *p)+((size_t) *(p+2))+
         ((size_t) *r)+((size_t) *(r+2))+2) >> 2);
-      q+=2;
-      p+=2;
-      r+=2;
+      q+=(ptrdiff_t) 2;
+      p+=(ptrdiff_t) 2;
+      r+=(ptrdiff_t) 2;
     }
     *q++=(unsigned char) ((((size_t) *p++)+((size_t) *r++)+1) >> 1);
     *q++=(unsigned char) ((((size_t) *p++)+((size_t) *r++)+1) >> 1);
@@ -502,19 +517,11 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MemoryInfo
     *pixel_info;
 
-  ssize_t
-    i,
-    y;
-
   PixelPacket
     *q;
 
-  unsigned char
-    *c1,
-    *c2,
-    *yy;
-
   size_t
+    extent,
     height,
     number_images,
     number_pixels,
@@ -524,13 +531,18 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   ssize_t
     count,
-    x;
+    i,
+    x,
+    y;
 
   unsigned char
+    *c1,
+    *c2,
     *chroma1,
     *chroma2,
     *header,
-    *luma;
+    *luma,
+    *yy;
 
   unsigned int
     overview;
@@ -628,11 +640,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate luma and chroma memory.
   */
-  pixel_info=AcquireVirtualMemory(image->columns+1UL,30*image->rows*
-    sizeof(*luma));
+  if (HeapOverflowSanityCheckGetSize(image->columns+1UL,image->rows,&extent) != MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if (HeapOverflowSanityCheckGetSize(extent,10,&number_pixels) != MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if (HeapOverflowSanityCheckGetSize(extent,30,&extent) != MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  pixel_info=AcquireVirtualMemory(extent,sizeof(*luma));
   if (pixel_info == (MemoryInfo *) NULL)
     ThrowPCDException(ResourceLimitError,"MemoryAllocationFailed");
-  number_pixels=(image->columns+1UL)*10*image->rows*sizeof(*luma);
   luma=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
   chroma1=(unsigned char *) GetVirtualMemoryBlob(pixel_info)+number_pixels;
   chroma2=(unsigned char *) GetVirtualMemoryBlob(pixel_info)+2*number_pixels;
@@ -777,8 +793,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image->rows=1024;
       for (i=0; i < (4*0x800); i++)
         (void) ReadBlobByte(image);
-      status=DecodeImage(image,luma,chroma1,chroma2);
-      if ((scene >= 5) && status)
+      status=DecodeImage(image,luma,chroma1,chroma2,&image->exception);
+      if ((scene >= 5) && (status != MagickFalse))
         {
           /*
             Recover luminance deltas for 3072x2048 image.
@@ -789,7 +805,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->rows=2048;
           offset=TellBlob(image)/0x800+12;
           offset=SeekBlob(image,offset*0x800,SEEK_SET);
-          status=DecodeImage(image,luma,chroma1,chroma2);
+          status=DecodeImage(image,luma,chroma1,chroma2,&image->exception);
           if ((scene >= 6) && (status != MagickFalse))
             {
               /*
