@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -269,15 +269,12 @@ static MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 */
 static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  unsigned char
-    attributes,
-    tag[3];
-
   Image
     *image;
 
   IndexPacket
-    index;
+    index,
+    *indexes;
 
   MagickBooleanType
     status;
@@ -288,18 +285,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   PDBInfo
     pdb_info;
 
-  IndexPacket
-    *indexes;
-
-  ssize_t
-    x;
-
   PixelPacket
     *q;
 
-  unsigned char
-    *p;
-
   size_t
     bits_per_pixel,
     num_pad_bytes,
@@ -310,10 +298,14 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count,
     img_offset,
     comment_offset = 0,
+    x,
     y;
 
   unsigned char
-    *pixels;
+    attributes,
+    *p,
+    *pixels,
+    tag[3];
 
   /*
     Open image file.
@@ -403,8 +395,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image structure.
   */
-  image->columns=(size_t) pdb_image.width;
-  image->rows=(size_t) pdb_image.height;
+  image->columns=(size_t) MagickAbsoluteValue(pdb_image.width);
+  image->rows=(size_t) MagickAbsoluteValue(pdb_image.height);
   image->depth=8;
   image->storage_class=PseudoClass;
   bits_per_pixel=pdb_image.type == 0 ? 2UL : pdb_image.type == 2 ? 4UL : 1UL;
@@ -629,13 +621,16 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         *p=c;
         c=ReadBlobByte(image);
       }
-      *p='\0';
       if (comment == (char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      *p='\0';
       (void) SetImageProperty(image,"comment",comment);
       comment=DestroyString(comment);
     }
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -1012,6 +1007,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   runlength=(unsigned char *) RelinquishMagickMemory(runlength);
   if (comment != (const char *) NULL)
     (void) WriteBlobString(image,comment);
-  (void) CloseBlob(image);
-  return(MagickTrue);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  return(status);
 }
